@@ -1,5 +1,6 @@
+# $Id: encoding.pm,v 1.43 2003/03/09 17:32:43 dankogai Exp $
 package encoding;
-our $VERSION = do { my @r = (q$Revision: 1.42 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+our $VERSION = do { my @r = (q$Revision: 1.43 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 use Encode;
 use strict;
@@ -63,6 +64,7 @@ sub import {
 		       });
 	};
     }	$DEBUG and warn "Filter installed";
+    defined ${^UNICODE} and ${^UNICODE} != 0 and return 1;
     for my $h (qw(STDIN STDOUT)){
 	if ($arg{$h}){
 	    unless (defined find_encoding($arg{$h})) {
@@ -144,6 +146,25 @@ new feature of Perl 5.6.
 Rewind to the future: starting from perl 5.8.0 with the B<encoding>
 pragma, you can write your script in any encoding you like (so long
 as the C<Encode> module supports it) and still enjoy Unicode support.
+This pragma achieves that by doing the followings;
+
+=over
+
+=item *
+
+Internally converts all literals (C<q//,qq//,qr//,qw///, qx//>) from
+the encoding specified to utf8.  In Perl 5.8.1 and later, literals in
+C<tr///> and C<DATA> pseudo-filehandle are also converted.
+
+=item *
+
+Changing PerlIO layers of C<STDIN> and C<STDOUT> to the encoding
+ specified.
+
+=back
+
+=head2 Literal Conversions
+
 You can write code in EUC-JP as follows:
 
   my $Rakuda = "\xF1\xD1\xF1\xCC"; # Camel in Kanji
@@ -156,7 +177,9 @@ the code in UTF-8:
   my $Rakuda = "\x{99F1}\x{99DD}"; # two Unicode Characters
   s/\bCamel\b/$Rakuda/;
 
-The B<encoding> pragma also modifies the filehandle disciplines of
+=head2 PerlIO layers for C<STD(IN|OUT)>
+
+The B<encoding> pragma also modifies the filehandle layers of
 STDIN and STDOUT to the specified encoding.  Therefore,
 
   use encoding "euc-jp";
@@ -170,23 +193,52 @@ not "\x{99F1}\x{99DD} is the symbol of perl.\n".
 
 You can override this by giving extra arguments; see below.
 
+=head1 FEATURES THAT REQUIRE 5.8.1
+
+Some of the features offered by this pragma requires perl 5.8.1.  Most
+of these works are by Inaba Hirohito.  Any other features and changes
+are good for 5.8.0.
+
+=over
+
+=item "NON-EUC" doublebyte encodings
+
+Because perl needs to parse script before applying this pragma, Such
+encodings as Shift_JIS and Big-5 that may contain '\' (BACKSLASH;
+\x5c) in the second byte fails because the second byte may
+accidentally escapes the quoting character that follows.  Perl 5.8.1
+or later fixes this problem.
+
+=item tr// 
+
+C<tr//> was overlooked by Perl 5 porters when they released perl 5.8.0
+See the section below for details.
+
+=item DATA pseudo-filehandle
+
+Another feature that was overlooked was C<DATA>. 
+
+=back
+
 =head1 USAGE
 
 =over 4
 
 =item use encoding [I<ENCNAME>] ;
 
-Sets the script encoding to I<ENCNAME>. Filehandle disciplines of
-STDIN and STDOUT are set to ":encoding(I<ENCNAME>)".  Note that STDERR
-will not be changed.
+Sets the script encoding to I<ENCNAME>.  And unless ${^UNICODE} 
+exists and non-zero, PerlIO layers of STDIN and STDOUT are set to
+":encoding(I<ENCNAME>)".
+
+Note that STDERR WILL NOT be changed.
+
+Also note that non-STD file handles remain unaffected.  Use C<use
+open> or C<binmode> to change layers of those.
 
 If no encoding is specified, the environment variable L<PERL_ENCODING>
 is consulted.  If no encoding can be found, the error C<Unknown encoding
 'I<ENCNAME>'> will be thrown.
 
-Note that non-STD file handles remain unaffected.  Use C<use open> or
-C<binmode> to change disciplines of those.
-
 =item use encoding I<ENCNAME> [ STDIN =E<gt> I<ENCNAME_IN> ...] ;
 
 You can also individually set encodings of STDIN and STDOUT via the
@@ -194,16 +246,21 @@ C<< STDIN => I<ENCNAME> >> form.  In this case, you cannot omit the
 first I<ENCNAME>.  C<< STDIN => undef >> turns the IO transcoding
 completely off.
 
+When ${^UNICODE} exists and non-zero, these options will completely
+ignored.  ${^UNICODE} is a variable introduced in perl 5.8.1.  See
+L<perlrun> see L<perlvar/"${^UNICODE}"> and L<perlrun/"-C"> for
+details (perl 5.8.1 and later).
+
 =item use encoding I<ENCNAME> Filter=E<gt>1;
 
 This turns the encoding pragma into a source filter.  While the
 default approach just decodes interpolated literals (in qq() and
 qr()), this will apply a source filter to the entire source code.  See
-L</"The Filter Option"> below for details
+L</"The Filter Option"> below for details.
 
 =item no encoding;
 
-Unsets the script encoding. The disciplines of STDIN, STDOUT are
+Unsets the script encoding. The layers of STDIN, STDOUT are
 reset to ":raw" (the default unprocessed raw stream of bytes).
 
 =back
@@ -215,7 +272,6 @@ identifiers.  In order to make C<${"\x{4eba}"}++> ($human++, where human
 is a single Han ideograph) work, you still need to write your script
 in UTF-8 -- or use a source filter.  That's what 'Filter=>1' does.
 
-
 What does this mean?  Your source code behaves as if it is written in
 UTF-8 with 'use utf8' in effect.  So even if your editor only supports
 Shift_JIS, for example, you can still try examples in Chapter 15 of
@@ -253,11 +309,29 @@ B<the whole script>.  However, the <no encoding> pragma is supported and
 B<use encoding> can appear as many times as you want in a given script. 
 The multiple use of this pragma is discouraged.
 
-Because of this nature, the use of this pragma inside the module is
-strongly discouraged (because the influence of this pragma lasts not
-only for the module but the script that uses).  But if you have to,
-make sure you say C<no encoding> at the end of the module so you
-contain the influence of the pragma within the module.
+Because of this nature -- the influence of this pragma lasts not only
+for the module but the script that uses the use of this pragma inside
+--, it is not recommended that you use this pragma inside modules.
+
+If you still have to write a module with this pragma, be very careful
+of the load order.  See the codes below;
+
+  # called module
+  package Module_IN_BAR;
+  use encoding "bar";
+  # stuff in "bar" encoding here
+  1;
+
+  # caller script
+  use encoding "foo"
+  use Module_IN_BAR;
+  # surprise! use encoding "bar" is in effect.
+
+The best way to avoid this oddity is to use this pragma RIGHT AFTER
+other modules are loaded.  i.e.
+
+  use Module_IN_BAR;
+  use encoding "foo";
 
 =head2 DO NOT MIX MULTIPLE ENCODINGS
 
@@ -292,33 +366,6 @@ resort to \x{....} just to spell your name in a native encoding.
 So feel free to put your strings in your encoding in quotes and
 regexes.
 
-=head2 format doesn't work well
-
-This pragma doesn't work well with format because PerlIO does not
-get along very well with it.  When format contains non-ascii
-characters it prints funny or gets "wide character warnings".
-To understand it, try the code below.
-
-  # Save this one in utf8
-  # replace *non-ascii* with a non-ascii string
-  my $camel;
-  format STDOUT =
-  *non-ascii*@>>>>>>>
-  $camel
-  .
-  $camel = "*non-ascii*";
-  binmode(STDOUT=>':encoding(utf8)'); # bang!
-  write;              # funny 
-  print $camel, "\n"; # fine
-
-Without binmode this happens to work but without binmode, print()
-fails instead of write().
-
-At any rate, the very use of format is questionable when it comes to
-unicode characters since you have to consider such things as character
-width (i.e. double-width for ideographs) and directions (i.e. BIDI for
-Arabic and Hebrew).
-
 =head2 tr/// with ranges
 
 The B<encoding> pragma works by decoding string literals in
@@ -347,8 +394,7 @@ Does not work as
 
 =back
 
-This counterintuitive behavior has been fixed in perl 5.8.1 and up
-by INABA Hirohito.
+This counterintuitive behavior has been fixed in perl 5.8.1.
 
 =head3 workaround to tr///;
 
@@ -366,7 +412,7 @@ is the same as classic idiom that makes C<tr///> 'interpolate'.
 
 Nevertheless, in case of B<encoding> pragma even C<q//> is affected so
 C<tr///> not being decoded was obviously against the will of Perl5
-Porters so it has been fixed.
+Porters so it has been fixed in Perl 5.8.1 or later.
 
 =head1 EXAMPLE - Greekperl
 
@@ -405,20 +451,52 @@ Porters so it has been fixed.
 
 =over
 
-=item *
+=item literals in regex that are logner than 127 bytes
 
 For native multibyte encodings (either fixed or variable length),
 the current implementation of the regular expressions may introduce
 recoding errors for regular expression literals longer than 127 bytes.
 
-=item *
+=item EBCDIC
 
 The encoding pragma is not supported on EBCDIC platforms.
 (Porters who are willing and able to remove this limitation are
 welcome.)
 
+=item format
+
+This pragma doesn't work well with format because PerlIO does not
+get along very well with it.  When format contains non-ascii
+characters it prints funny or gets "wide character warnings".
+To understand it, try the code below.
+
+  # Save this one in utf8
+  # replace *non-ascii* with a non-ascii string
+  my $camel;
+  format STDOUT =
+  *non-ascii*@>>>>>>>
+  $camel
+  .
+  $camel = "*non-ascii*";
+  binmode(STDOUT=>':encoding(utf8)'); # bang!
+  write;              # funny 
+  print $camel, "\n"; # fine
+
+Without binmode this happens to work but without binmode, print()
+fails instead of write().
+
+At any rate, the very use of format is questionable when it comes to
+unicode characters since you have to consider such things as character
+width (i.e. double-width for ideographs) and directions (i.e. BIDI for
+Arabic and Hebrew).
+
 =back
 
+=head1 HISTORY
+
+This pragma first appeared in Perl 5.8.0.  For features that require 
+5.8.1 and better, see above.
+
 =head1 SEE ALSO
 
 L<perlunicode>, L<Encode>, L<open>, L<Filter::Util::Call>,
